@@ -149,6 +149,10 @@ ShaderGenerator::ShaderGenerator() :
     {
         mShaderLanguage = "hlsl";
     }
+    else if (hmgr.isLanguageSupported("glslang"))
+    {
+        mShaderLanguage = "glslang";
+    }
     else
     {
         mShaderLanguage = "null"; // falling back to HLSL, for unit tests mainly
@@ -262,6 +266,10 @@ void ShaderGenerator::createBuiltinSRSFactories()
         addSubRenderStateFactory(curFactory);
         mBuiltinSRSFactories.push_back(curFactory);
 
+        curFactory = new CookTorranceLightingFactory;
+        addSubRenderStateFactory(curFactory);
+        mBuiltinSRSFactories.push_back(curFactory);
+
         curFactory = OGRE_NEW IntegratedPSSM3Factory;   
         addSubRenderStateFactory(curFactory);
         mBuiltinSRSFactories.push_back(curFactory);
