@@ -36,7 +36,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Material::Material(ResourceManager* creator, const String& name, ResourceHandle handle,
         const String& group, bool isManual, ManualResourceLoader* loader)
-        :Resource(creator, name, handle, group, isManual, loader),
+        :Resource(creator, name, handle, group, false, NULL),
          mReceiveShadows(true),
          mTransparencyCastsShadows(false),
          mCompilationRequired(true)
@@ -44,8 +44,7 @@ namespace Ogre {
         // Override isManual, not applicable for Material (we always want to call loadImpl)
         if(isManual)
         {
-            mIsManual = false;
-            LogManager::getSingleton().logMessage("Material " + name + 
+            LogManager::getSingleton().logWarning("Material " + name +
                 " was requested with isManual=true, but this is not applicable " 
                 "for materials; the flag has been reset to false");
         }
@@ -75,19 +74,10 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Material& Material::operator=(const Material& rhs)
     {
-        mName = rhs.mName;
-        mGroup = rhs.mGroup;
-        mCreator = rhs.mCreator;
-        mIsManual = rhs.mIsManual;
-        mLoader = rhs.mLoader;
-        mHandle = rhs.mHandle;
-        mSize = rhs.mSize;
+        Resource::operator=(rhs);
         mReceiveShadows = rhs.mReceiveShadows;
         mTransparencyCastsShadows = rhs.mTransparencyCastsShadows;
 
-        mLoadingState.store(rhs.mLoadingState.load());
-        mIsBackgroundLoaded = rhs.mIsBackgroundLoaded;
-
         // Copy Techniques
         this->removeAllTechniques();
         Techniques::const_iterator i, iend;
@@ -168,30 +158,23 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     size_t Material::calculateSize(void) const
     {
-        size_t memSize = 0;
+        size_t memSize = sizeof(*this) + Resource::calculateSize();
 
         // Tally up techniques
-        Techniques::const_iterator i, iend;
-        iend = mTechniques.end();
-        for (i = mTechniques.begin(); i != iend; ++i)
+        for (auto t : mTechniques)
         {
-            memSize += (*i)->calculateSize();
+            memSize += t->calculateSize();
         }
 
-        memSize += sizeof(bool) * 3;
         memSize += mUnsupportedReasons.size() * sizeof(char);
-        memSize += sizeof(LodStrategy);
-
-        memSize += Resource::calculateSize();
 
         return memSize;
     }
     //-----------------------------------------------------------------------
-    MaterialPtr Material::clone(const String& newName, bool changeGroup, 
-        const String& newGroup) const
+    MaterialPtr Material::clone(const String& newName, const String& newGroup) const
     {
         MaterialPtr newMat =
-            MaterialManager::getSingleton().create(newName, changeGroup ? newGroup : mGroup);
+            MaterialManager::getSingleton().create(newName, newGroup.empty() ? mGroup : newGroup);
 
         if(!newMat) // interception by collision handler
             return newMat;
@@ -201,7 +184,7 @@ namespace Ogre {
         // Assign values from this
         *newMat = *this;
         // Restore new group if required, will have been overridden by operator
-        if (changeGroup)
+        if (!newGroup.empty())
         {
             newMat->mGroup = newGroup;
         }
@@ -210,6 +193,14 @@ namespace Ogre {
         newMat->mName = newName;
         newMat->mHandle = newHandle;
 
+        //if we're cloning from a loaded material, notify the creator or otherwise size won't be right
+        if (newMat->getLoadingState() == LOADSTATE_LOADED)
+        {
+            // Notify manager
+            if (mCreator)
+                mCreator->_notifyResourceLoaded(newMat.get());
+        }
+
         return newMat;
     }
     //-----------------------------------------------------------------------
@@ -219,17 +210,12 @@ namespace Ogre {
         ResourceHandle savedHandle = mat->mHandle;
         String savedName = mat->mName;
         String savedGroup = mat->mGroup;
-        ManualResourceLoader* savedLoader = mat->mLoader;
-        bool savedManual = mat->mIsManual;
         // Assign values from this
         *mat = *this;
         // Correct the name & handle, they get copied too
         mat->mName = savedName;
         mat->mHandle = savedHandle;
         mat->mGroup = savedGroup;
-        mat->mIsManual = savedManual;
-        mat->mLoader = savedLoader;
-
     }
     //-----------------------------------------------------------------------
     void Material::applyDefaults(void)
@@ -242,15 +228,11 @@ namespace Ogre {
             String savedName = mName;
             String savedGroup = mGroup;
             ResourceHandle savedHandle = mHandle;
-            ManualResourceLoader *savedLoader = mLoader;
-            bool savedManual = mIsManual;
             *this = *defaults;
             // restore name & handle
             mName = savedName;
             mHandle = savedHandle;
             mGroup = savedGroup;
-            mLoader = savedLoader;
-            mIsManual = savedManual;
         }
         mCompilationRequired = true;
 
@@ -264,12 +246,6 @@ namespace Ogre {
         return t;
     }
     //-----------------------------------------------------------------------
-    Technique* Material::getTechnique(unsigned short index) const
-    {
-        assert (index < mTechniques.size() && "Index out of bounds.");
-        return mTechniques[index];
-    }
-    //-----------------------------------------------------------------------
     Technique* Material::getTechnique(const String& name) const
     {
         Techniques::const_iterator i    = mTechniques.begin();
@@ -289,22 +265,6 @@ namespace Ogre {
 
         return foundTechnique;
     }
-    //-----------------------------------------------------------------------   
-    unsigned short Material::getNumTechniques(void) const
-    {
-        return static_cast<unsigned short>(mTechniques.size());
-    }
-    //-----------------------------------------------------------------------
-    Technique* Material::getSupportedTechnique(unsigned short index)
-    {
-        assert (index < mSupportedTechniques.size() && "Index out of bounds.");
-        return mSupportedTechniques[index];
-    }
-    //-----------------------------------------------------------------------   
-    unsigned short Material::getNumSupportedTechniques(void) const
-    {
-        return static_cast<unsigned short>(mSupportedTechniques.size());
-    }
     //-----------------------------------------------------------------------
     unsigned short Material::getNumLodLevels(unsigned short schemeIndex) const
     {
@@ -321,7 +281,7 @@ namespace Ogre {
             i = mBestTechniquesBySchemeList.begin();
         }
 
-        return static_cast<unsigned short>(i->second->size());
+        return static_cast<unsigned short>(i->second.size());
     }
     //-----------------------------------------------------------------------
     unsigned short Material::getNumLodLevels(const String& schemeName) const
@@ -335,22 +295,10 @@ namespace Ogre {
         mSupportedTechniques.push_back(t);
         // get scheme
         unsigned short schemeIndex = t->_getSchemeIndex();
-        BestTechniquesBySchemeList::iterator i =
-            mBestTechniquesBySchemeList.find(schemeIndex);
-        LodTechniques* lodtechs = 0;
-        if (i == mBestTechniquesBySchemeList.end())
-        {
-            lodtechs = OGRE_NEW_T(LodTechniques, MEMCATEGORY_RESOURCE);
-            mBestTechniquesBySchemeList[schemeIndex] = lodtechs;
-        }
-        else
-        {
-            lodtechs = i->second;
-        }
 
         // Insert won't replace if supported technique for this scheme/lod is
         // already there, which is what we want
-        lodtechs->emplace(t->getLodIndex(), t);
+        mBestTechniquesBySchemeList[schemeIndex].emplace(t->getLodIndex(), t);
 
     }
     //-----------------------------------------------------------------------------
@@ -382,13 +330,13 @@ namespace Ogre {
             }
 
             // get LOD
-            LodTechniques::iterator li = si->second->find(lodIndex);
+            LodTechniques::iterator li = si->second.find(lodIndex);
             // LOD not found? 
-            if (li == si->second->end())
+            if (li == si->second.end())
             {
                 // Use the next LOD level up
-                for (LodTechniques::reverse_iterator rli = si->second->rbegin(); 
-                    rli != si->second->rend(); ++rli)
+                for (LodTechniques::reverse_iterator rli = si->second.rbegin();
+                    rli != si->second.rend(); ++rli)
                 {
                     if (rli->second->getLodIndex() < lodIndex)
                     {
@@ -401,7 +349,7 @@ namespace Ogre {
                 {
                     // shouldn't ever hit this really, unless user defines no LOD 0
                     // pick the first LOD we have (must be at least one to have a scheme entry)
-                    ret = si->second->begin()->second;
+                    ret = si->second.begin()->second;
                 }
 
             }
@@ -508,11 +456,6 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Material::clearBestTechniqueList(void)
     {
-        for (BestTechniquesBySchemeList::iterator i = mBestTechniquesBySchemeList.begin();
-            i != mBestTechniquesBySchemeList.end(); ++i)
-        {
-            OGRE_DELETE_T(i->second, LodTechniques, MEMCATEGORY_RESOURCE);
-        }
         mBestTechniquesBySchemeList.clear();
     }
     //-----------------------------------------------------------------------
@@ -811,23 +754,6 @@ namespace Ogre {
     {
         return LodValueIterator(mUserLodValues.begin(), mUserLodValues.end());
     }
-
-    //-----------------------------------------------------------------------
-    bool Material::applyTextureAliases(const AliasTextureNamePairList& aliasList, const bool apply) const
-    {
-        // iterate through all techniques and apply texture aliases
-        Techniques::const_iterator i, iend;
-        iend = mTechniques.end();
-        bool testResult = false;
-
-        for (i = mTechniques.begin(); i != iend; ++i)
-        {
-            if ((*i)->applyTextureAliases(aliasList, apply))
-                testResult = true;
-        }
-
-        return testResult;
-    }
     //---------------------------------------------------------------------
     const LodStrategy *Material::getLodStrategy() const
     {
