@@ -84,13 +84,6 @@ SceneManager::ShadowRenderer::~ShadowRenderer() {}
 void SceneManager::ShadowRenderer::setShadowColour(const ColourValue& colour)
 {
     mShadowColour = colour;
-
-    // Change shadow material setting only when it's prepared,
-    // otherwise, it'll set up while preparing shadow materials.
-    if (mShadowModulativePass)
-    {
-        mShadowModulativePass->getFragmentProgramParameters()->setNamedConstant("shadowColor",colour);
-    }
 }
 
 void SceneManager::ShadowRenderer::render(RenderQueueGroup* group,
@@ -134,6 +127,7 @@ void SceneManager::ShadowRenderer::renderAdditiveStencilShadowedQueueGroupObject
     QueuedRenderableCollection::OrganisationMode om)
 {
     LightList lightList;
+    auto visitor = mSceneManager->getQueuedRenderableVisitor();
 
     for (const auto& pg : pGroup->getPriorityGroups())
     {
@@ -146,9 +140,9 @@ void SceneManager::ShadowRenderer::renderAdditiveStencilShadowedQueueGroupObject
         lightList.clear();
 
         // Render all the ambient passes first, no light iteration, no lights
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsBasic(), om, false, false, &lightList);
+        visitor->renderObjects(pPriorityGrp->getSolidsBasic(), om, false, false, &lightList);
         // Also render any objects which have receive shadows disabled
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, true, true);
 
 
         // Now iterate per light
@@ -176,18 +170,18 @@ void SceneManager::ShadowRenderer::renderAdditiveStencilShadowedQueueGroupObject
                 // Clear stencil
                 mDestRenderSystem->clearFrameBuffer(FBT_STENCIL);
                 renderShadowVolumesToStencil(l, mSceneManager->mCameraInProgress, false);
-                // turn stencil check on
-                mDestRenderSystem->setStencilCheckEnabled(true);
                 // NB we render where the stencil is equal to zero to render lit areas
-                mDestRenderSystem->setStencilBufferParams(CMPF_EQUAL, 0);
+                StencilState stencilState;
+                stencilState.enabled = true;
+                stencilState.compareOp = CMPF_EQUAL;
+                mDestRenderSystem->setStencilState(stencilState);
             }
 
             // render lighting passes for this light
-            mSceneManager->renderObjects(pPriorityGrp->getSolidsDiffuseSpecular(), om, false, false, &lightList);
+            visitor->renderObjects(pPriorityGrp->getSolidsDiffuseSpecular(), om, false, false, &lightList);
 
             // Reset stencil params
-            mDestRenderSystem->setStencilBufferParams();
-            mDestRenderSystem->setStencilCheckEnabled(false);
+            mDestRenderSystem->setStencilState(StencilState());
 
             if (scissored == CLIPPED_SOME)
                 mSceneManager->resetScissor();
@@ -198,7 +192,7 @@ void SceneManager::ShadowRenderer::renderAdditiveStencilShadowedQueueGroupObject
 
 
         // Now render decal passes, no need to set lights as lighting will be disabled
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsDecal(), om, false, false);
+        visitor->renderObjects(pPriorityGrp->getSolidsDecal(), om, false, false);
 
 
     }// for each priority
@@ -209,9 +203,9 @@ void SceneManager::ShadowRenderer::renderAdditiveStencilShadowedQueueGroupObject
         RenderPriorityGroup* pPriorityGrp = pg.second;
 
         // Do unsorted transparents
-        mSceneManager->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, true, true);
         // Do transparents (always descending sort)
-        mSceneManager->renderObjects(pPriorityGrp->getTransparents(),
+        visitor->renderObjects(pPriorityGrp->getTransparents(),
             QueuedRenderableCollection::OM_SORT_DESCENDING, true, true);
 
     }// for each priority
@@ -232,6 +226,7 @@ void SceneManager::ShadowRenderer::renderModulativeStencilShadowedQueueGroupObje
     interleaved as in the normal rendering loop.
     */
     // Iterate through priorities
+    auto visitor = mSceneManager->getQueuedRenderableVisitor();
     for (const auto& pg : pGroup->getPriorityGroups())
     {
         RenderPriorityGroup* pPriorityGrp = pg.second;
@@ -240,7 +235,7 @@ void SceneManager::ShadowRenderer::renderModulativeStencilShadowedQueueGroupObje
         pPriorityGrp->sort(mSceneManager->mCameraInProgress);
 
         // Do (shadowable) solids
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsBasic(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getSolidsBasic(), om, true, true);
     }
 
     // Override auto param ambient to force vertex programs to use shadow colour
@@ -257,14 +252,14 @@ void SceneManager::ShadowRenderer::renderModulativeStencilShadowedQueueGroupObje
             renderShadowVolumesToStencil(l, mSceneManager->mCameraInProgress, true);
             // render full-screen shadow modulator for all lights
             mSceneManager->_setPass(mShadowModulativePass);
-            // turn stencil check on
-            mDestRenderSystem->setStencilCheckEnabled(true);
             // NB we render where the stencil is not equal to zero to render shadows, not lit areas
-            mDestRenderSystem->setStencilBufferParams(CMPF_NOT_EQUAL, 0);
+            StencilState stencilState;
+            stencilState.enabled = true;
+            stencilState.compareOp = CMPF_NOT_EQUAL;
+            mDestRenderSystem->setStencilState(stencilState);
             mSceneManager->renderSingleObject(mFullScreenQuad, mShadowModulativePass, false, false);
             // Reset stencil params
-            mDestRenderSystem->setStencilBufferParams();
-            mDestRenderSystem->setStencilCheckEnabled(false);
+            mDestRenderSystem->setStencilState(StencilState());
         }
 
     }// for each light
@@ -278,7 +273,7 @@ void SceneManager::ShadowRenderer::renderModulativeStencilShadowedQueueGroupObje
         RenderPriorityGroup* pPriorityGrp = pg.second;
 
         // Do non-shadowable solids
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, true, true);
 
     }// for each priority
 
@@ -289,9 +284,9 @@ void SceneManager::ShadowRenderer::renderModulativeStencilShadowedQueueGroupObje
         RenderPriorityGroup* pPriorityGrp = pg.second;
 
         // Do unsorted transparents
-        mSceneManager->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, true, true);
         // Do transparents (always descending sort)
-        mSceneManager->renderObjects(pPriorityGrp->getTransparents(),
+        visitor->renderObjects(pPriorityGrp->getTransparents(),
             QueuedRenderableCollection::OM_SORT_DESCENDING, true, true);
 
     }// for each priority
@@ -322,6 +317,7 @@ void SceneManager::ShadowRenderer::renderTextureShadowCasterQueueGroupObjects(
         mSceneManager->setAmbientLight(mShadowColour);
     }
 
+    auto visitor = mSceneManager->getQueuedRenderableVisitor();
     for (const auto& pg : pGroup->getPriorityGroups())
     {
         RenderPriorityGroup* pPriorityGrp = pg.second;
@@ -330,12 +326,12 @@ void SceneManager::ShadowRenderer::renderTextureShadowCasterQueueGroupObjects(
         pPriorityGrp->sort(mSceneManager->mCameraInProgress);
 
         // Do solids, override light list incase any vertex programs use them
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsBasic(), om, false, false, &mShadowTextureCurrentCasterLightList);
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, false, false, &mShadowTextureCurrentCasterLightList);
+        visitor->renderObjects(pPriorityGrp->getSolidsBasic(), om, false, false, &mShadowTextureCurrentCasterLightList);
+        visitor->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, false, false, &mShadowTextureCurrentCasterLightList);
         // Do unsorted transparents that cast shadows
-        mSceneManager->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, false, false, &mShadowTextureCurrentCasterLightList);
+        visitor->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, false, false, &mShadowTextureCurrentCasterLightList);
         // Do transparents that cast shadows
-        mSceneManager->renderObjects(pPriorityGrp->getTransparents(), QueuedRenderableCollection::OM_SORT_DESCENDING,
+        visitor->renderObjects(pPriorityGrp->getTransparents(), QueuedRenderableCollection::OM_SORT_DESCENDING,
                       false, false, &mShadowTextureCurrentCasterLightList, true);
 
     }// for each priority
@@ -357,6 +353,7 @@ void SceneManager::ShadowRenderer::renderModulativeTextureShadowedQueueGroupObje
     interleaved as in the normal rendering loop.
     */
     // Iterate through priorities
+    auto visitor = mSceneManager->getQueuedRenderableVisitor();
     for (const auto& pg : pGroup->getPriorityGroups())
     {
         RenderPriorityGroup* pPriorityGrp = pg.second;
@@ -365,8 +362,8 @@ void SceneManager::ShadowRenderer::renderModulativeTextureShadowedQueueGroupObje
         pPriorityGrp->sort(mSceneManager->mCameraInProgress);
 
         // Do solids
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsBasic(), om, true, true);
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getSolidsBasic(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, true, true);
     }
 
 
@@ -408,7 +405,7 @@ void SceneManager::ShadowRenderer::renderModulativeTextureShadowedQueueGroupObje
                 TextureUnitState* t = NULL;
                 // Add spot fader if not present already
                 if (targetPass->getNumTextureUnitStates() == 2 &&
-                    targetPass->getTextureUnitState(1)->getTextureName() == "spot_shadow_fade.dds")
+                    targetPass->getTextureUnitState(1)->_getTexturePtr() == mSpotFadeTexture)
                 {
                     // Just set
                     t = targetPass->getTextureUnitState(1);
@@ -419,7 +416,8 @@ void SceneManager::ShadowRenderer::renderModulativeTextureShadowedQueueGroupObje
                     while(targetPass->getNumTextureUnitStates() > 1)
                         targetPass->removeTextureUnitState(1);
 
-                    t = targetPass->createTextureUnitState("spot_shadow_fade.dds");
+                    t = targetPass->createTextureUnitState();
+                    t->setTexture(mSpotFadeTexture);
                     t->setColourOperation(LBO_ADD);
                     t->setTextureAddressingMode(TextureUnitState::TAM_CLAMP);
                 }
@@ -471,9 +469,9 @@ void SceneManager::ShadowRenderer::renderModulativeTextureShadowedQueueGroupObje
         RenderPriorityGroup* pPriorityGrp = pg.second;
 
         // Do unsorted transparents
-        mSceneManager->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, true, true);
         // Do transparents (always descending)
-        mSceneManager->renderObjects(pPriorityGrp->getTransparents(),
+        visitor->renderObjects(pPriorityGrp->getTransparents(),
             QueuedRenderableCollection::OM_SORT_DESCENDING, true, true);
 
     }// for each priority
@@ -485,6 +483,7 @@ void SceneManager::ShadowRenderer::renderAdditiveTextureShadowedQueueGroupObject
     QueuedRenderableCollection::OrganisationMode om)
 {
     LightList lightList;
+    auto visitor = mSceneManager->getQueuedRenderableVisitor();
 
     for (const auto& pg : pGroup->getPriorityGroups())
     {
@@ -497,9 +496,9 @@ void SceneManager::ShadowRenderer::renderAdditiveTextureShadowedQueueGroupObject
         lightList.clear();
 
         // Render all the ambient passes first, no light iteration, no lights
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsBasic(), om, false, false, &lightList);
+        visitor->renderObjects(pPriorityGrp->getSolidsBasic(), om, false, false, &lightList);
         // Also render any objects which have receive shadows disabled
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getSolidsNoShadowReceive(), om, true, true);
 
 
         // only perform this next part if we're in the 'normal' render stage, to avoid
@@ -567,7 +566,7 @@ void SceneManager::ShadowRenderer::renderAdditiveTextureShadowedQueueGroupObject
                 if(scissored == CLIPPED_ALL || clipped == CLIPPED_ALL)
                     continue;
 
-                mSceneManager->renderObjects(pPriorityGrp->getSolidsDiffuseSpecular(), om, false, false, &lightList);
+                visitor->renderObjects(pPriorityGrp->getSolidsDiffuseSpecular(), om, false, false, &lightList);
                 if (scissored == CLIPPED_SOME)
                     mSceneManager->resetScissor();
                 if (clipped == CLIPPED_SOME)
@@ -578,7 +577,7 @@ void SceneManager::ShadowRenderer::renderAdditiveTextureShadowedQueueGroupObject
             mSceneManager->mIlluminationStage = IRS_NONE;
 
             // Now render decal passes, no need to set lights as lighting will be disabled
-            mSceneManager->renderObjects(pPriorityGrp->getSolidsDecal(), om, false, false);
+            visitor->renderObjects(pPriorityGrp->getSolidsDecal(), om, false, false);
 
         }
 
@@ -591,9 +590,9 @@ void SceneManager::ShadowRenderer::renderAdditiveTextureShadowedQueueGroupObject
         RenderPriorityGroup* pPriorityGrp = pg.second;
 
         // Do unsorted transparents
-        mSceneManager->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, true, true);
+        visitor->renderObjects(pPriorityGrp->getTransparentsUnsorted(), om, true, true);
         // Do transparents (always descending sort)
-        mSceneManager->renderObjects(pPriorityGrp->getTransparents(),
+        visitor->renderObjects(pPriorityGrp->getTransparents(),
             QueuedRenderableCollection::OM_SORT_DESCENDING, true, true);
 
     }// for each priority
@@ -611,13 +610,14 @@ void SceneManager::ShadowRenderer::renderTextureShadowReceiverQueueGroupObjects(
     // Override auto param ambient to force vertex programs to go full-bright
     ColourValue currAmbient = mSceneManager->getAmbientLight();
     mSceneManager->setAmbientLight(ColourValue::White);
+    auto visitor = mSceneManager->getQueuedRenderableVisitor();
 
     for (const auto& pg : pGroup->getPriorityGroups())
     {
         RenderPriorityGroup* pPriorityGrp = pg.second;
 
         // Do solids, override light list incase any vertex programs use them
-        mSceneManager->renderObjects(pPriorityGrp->getSolidsBasic(), om, false, false, &nullLightList);
+        visitor->renderObjects(pPriorityGrp->getSolidsBasic(), om, false, false, &nullLightList);
 
         // Don't render transparents or passes which have shadow receipt disabled
 
@@ -923,15 +923,13 @@ void SceneManager::ShadowRenderer::renderShadowVolumesToStencil(const Light* lig
     // Do we have access to vertex programs?
     bool extrudeInSoftware = true;
     bool finiteExtrude = !mShadowUseInfiniteFarPlane;
-    if (ShadowVolumeExtrudeProgram::frgProgram->isSupported())
+    if (const auto& fprog = mShadowStencilPass->getFragmentProgram())
     {
         extrudeInSoftware = false;
         // attach the appropriate extrusion vertex program
         // Note we never unset it because support for vertex programs is constant
-        mShadowStencilPass->setGpuProgram(
-            GPT_VERTEX_PROGRAM, ShadowVolumeExtrudeProgram::get(light->getType(), finiteExtrude),
-            false);
-        mShadowStencilPass->setGpuProgram(GPT_FRAGMENT_PROGRAM, ShadowVolumeExtrudeProgram::frgProgram);
+        mShadowStencilPass->setGpuProgram(GPT_VERTEX_PROGRAM,
+                                          ShadowVolumeExtrudeProgram::get(light->getType(), finiteExtrude), false);
         // Set params
         if (finiteExtrude)
         {
@@ -943,11 +941,8 @@ void SceneManager::ShadowRenderer::renderShadowVolumesToStencil(const Light* lig
         }
         if (mDebugShadows)
         {
-            mShadowDebugPass->setGpuProgram(
-                GPT_VERTEX_PROGRAM,
-                ShadowVolumeExtrudeProgram::get(light->getType(), finiteExtrude, true), false);
-            mShadowDebugPass->setGpuProgram(GPT_FRAGMENT_PROGRAM, ShadowVolumeExtrudeProgram::frgProgram);
-
+            mShadowDebugPass->setGpuProgram(GPT_VERTEX_PROGRAM,
+                                            ShadowVolumeExtrudeProgram::get(light->getType(), finiteExtrude), false);
 
             // Set params
             if (finiteExtrude)
@@ -961,11 +956,7 @@ void SceneManager::ShadowRenderer::renderShadowVolumesToStencil(const Light* lig
         }
 
         mSceneManager->bindGpuProgram(mShadowStencilPass->getVertexProgram()->_getBindingDelegate());
-        if (ShadowVolumeExtrudeProgram::frgProgram)
-        {
-            mSceneManager->bindGpuProgram(mShadowStencilPass->getFragmentProgram()->_getBindingDelegate());
-        }
-
+        mSceneManager->bindGpuProgram(fprog->_getBindingDelegate());
     }
     else
     {
@@ -985,7 +976,6 @@ void SceneManager::ShadowRenderer::renderShadowVolumesToStencil(const Light* lig
     mDestRenderSystem->setColourBlendState(disabled);
     mDestRenderSystem->_disableTextureUnitsFrom(0);
     mDestRenderSystem->_setDepthBufferParams(true, false, CMPF_LESS);
-    mDestRenderSystem->setStencilCheckEnabled(true);
 
     // Figure out the near clip volume
     const PlaneBoundedVolume& nearClipVol =
@@ -1091,22 +1081,21 @@ void SceneManager::ShadowRenderer::renderShadowVolumesToStencil(const Light* lig
         if (mDebugShadows)
         {
             // reset stencil & colour ops
-            mDestRenderSystem->setStencilBufferParams();
-            if (mShadowDebugPass->hasFragmentProgram())
-            {
-                mShadowDebugPass->getFragmentProgramParameters()->setNamedConstant(
-                    "shadowColor", zfailAlgo ? ColourValue(0.7, 0.0, 0.2) : ColourValue(0.0, 0.7, 0.2));
-            }
+            mDestRenderSystem->setStencilState(StencilState());
+
+            auto shadowColour = mShadowColour;
+            mShadowColour = zfailAlgo ? ColourValue(0.7, 0.0, 0.2) : ColourValue(0.0, 0.7, 0.2);
             mSceneManager->_setPass(mShadowDebugPass);
             renderShadowVolumeObjects(shadowRenderables, mShadowDebugPass, &lightList, flags,
                 true, false, false);
             mDestRenderSystem->setColourBlendState(disabled);
             mDestRenderSystem->_setDepthBufferParams(true, false, CMPF_LESS);
+            mShadowColour = shadowColour;
         }
     }
 
     // revert stencil state
-    mDestRenderSystem->setStencilCheckEnabled(false);
+    mDestRenderSystem->setStencilState(StencilState());
 
     mDestRenderSystem->unbindGpuProgram(GPT_VERTEX_PROGRAM);
 
@@ -1221,34 +1210,23 @@ void SceneManager::ShadowRenderer::setShadowVolumeStencilState(bool secondpass,
     // When two-sided stencil, always pass front face stencil
     // operation parameters and the inverse of them will happen
     // for back faces
+    StencilState stencilState;
+    stencilState.enabled = true;
+    stencilState.compareOp = CMPF_ALWAYS_PASS; // always pass stencil check
+    stencilState.twoSidedOperation = twosided;
     if ( !twosided && ((secondpass || zfail) && !(secondpass && zfail)) )
     {
         mSceneManager->mPassCullingMode = twosided? CULL_NONE : CULL_ANTICLOCKWISE;
-        mDestRenderSystem->setStencilBufferParams(
-            CMPF_ALWAYS_PASS, // always pass stencil check
-            0, // no ref value (no compare)
-            0xFFFFFFFF, // no compare mask
-            0xFFFFFFFF, // no write mask
-            SOP_KEEP, // stencil test will never fail
-            zfail ? incrOp : SOP_KEEP, // back face depth fail
-            zfail ? SOP_KEEP : decrOp, // back face pass
-            twosided
-            );
+        stencilState.depthFailOp = zfail ? incrOp : SOP_KEEP; // back face depth fail
+        stencilState.depthStencilPassOp = zfail ? SOP_KEEP : decrOp; // back face pass
     }
     else
     {
         mSceneManager->mPassCullingMode = twosided? CULL_NONE : CULL_CLOCKWISE;
-        mDestRenderSystem->setStencilBufferParams(
-            CMPF_ALWAYS_PASS, // always pass stencil check
-            0, // no ref value (no compare)
-            0xFFFFFFFF, // no compare mask
-            0xFFFFFFFF, // no write mask
-            SOP_KEEP, // stencil test will never fail
-            zfail ? decrOp : SOP_KEEP, // front face depth fail
-            zfail ? SOP_KEEP : incrOp, // front face pass
-            twosided
-            );
+        stencilState.depthFailOp = zfail ? decrOp : SOP_KEEP; // front face depth fail
+        stencilState.depthStencilPassOp = zfail ? SOP_KEEP : incrOp; // front face pass
     }
+    mDestRenderSystem->setStencilState(stencilState);
     mDestRenderSystem->_setCullingMode(mSceneManager->mPassCullingMode);
 
 }
@@ -1319,10 +1297,14 @@ void SceneManager::ShadowRenderer::setShadowTechnique(ShadowTechnique technique)
         }
     }
 
+    if (mShadowTechnique == SHADOWTYPE_TEXTURE_MODULATIVE && !mSpotFadeTexture)
+        mSpotFadeTexture = TextureManager::getSingleton().load("spot_shadow_fade.dds", RGN_INTERNAL);
+
     if ((mShadowTechnique & SHADOWDETAILTYPE_TEXTURE) == 0)
     {
         // Destroy shadow textures to optimise resource usage
-        mSceneManager->destroyShadowTextures();
+        destroyShadowTextures();
+        mSpotFadeTexture.reset();
     }
     else
     {
@@ -1350,191 +1332,38 @@ void SceneManager::ShadowRenderer::initShadowVolumeMaterials()
 
     if (!mShadowDebugPass)
     {
-        MaterialPtr matDebug =
-            MaterialManager::getSingleton().getByName("Ogre/Debug/ShadowVolumes");
-        if (!matDebug)
-        {
-            // Create
-            matDebug = MaterialManager::getSingleton().create(
-                "Ogre/Debug/ShadowVolumes",
-                ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
-            mShadowDebugPass = matDebug->getTechnique(0)->getPass(0);
-            mShadowDebugPass->setSceneBlending(SBT_ADD);
-            mShadowDebugPass->setLightingEnabled(false);
-            mShadowDebugPass->setDepthWriteEnabled(false);
-            mShadowDebugPass->setCullingMode(CULL_NONE);
-
-            ShadowVolumeExtrudeProgram::initialise();
-            if (ShadowVolumeExtrudeProgram::frgProgram->isSupported())
-            {
-                // Enable the (infinite) point light extruder for now, just to get some params
-                mShadowDebugPass->setGpuProgram(
-                    GPT_VERTEX_PROGRAM, ShadowVolumeExtrudeProgram::get(Light::LT_POINT, false));
-                mShadowDebugPass->setGpuProgram(GPT_FRAGMENT_PROGRAM, ShadowVolumeExtrudeProgram::frgProgram);
-                msInfiniteExtrusionParams =
-                    mShadowDebugPass->getVertexProgramParameters();
-                msInfiniteExtrusionParams->setAutoConstant(0,
-                    GpuProgramParameters::ACT_WORLDVIEWPROJ_MATRIX);
-                msInfiniteExtrusionParams->setAutoConstant(4,
-                    GpuProgramParameters::ACT_LIGHT_POSITION_OBJECT_SPACE);
-                // Note ignored extra parameter - for compatibility with finite extrusion vertex program
-                msInfiniteExtrusionParams->setAutoConstant(5,
-                    GpuProgramParameters::ACT_SHADOW_EXTRUSION_DISTANCE);
-
-                try {
-                    msInfiniteExtrusionParams->setNamedAutoConstant(
-                        "worldviewproj_matrix",
-                        GpuProgramParameters::ACT_WORLDVIEWPROJ_MATRIX);
-                    msInfiniteExtrusionParams->setNamedAutoConstant(
-                        "light_position_object_space",
-                        GpuProgramParameters::ACT_LIGHT_POSITION_OBJECT_SPACE);
-                } catch(InvalidParametersException&) {} // ignore
-            }
-            matDebug->compile();
-
-        }
-        else
-        {
-            mShadowDebugPass = matDebug->getTechnique(0)->getPass(0);
-
-            if (mShadowDebugPass->isProgrammable())
-            {
-                msInfiniteExtrusionParams = mShadowDebugPass->getVertexProgramParameters();
-            }
-        }
+        ShadowVolumeExtrudeProgram::initialise();
+        MaterialPtr matDebug = MaterialManager::getSingleton().getByName("Ogre/Debug/ShadowVolumes");
+        mShadowDebugPass = matDebug->getTechnique(0)->getPass(0);
+        msInfiniteExtrusionParams = mShadowDebugPass->getVertexProgramParameters();
     }
 
     if (!mShadowStencilPass)
     {
-
-        MaterialPtr matStencil = MaterialManager::getSingleton().getByName(
-            "Ogre/StencilShadowVolumes");
-        if (!matStencil)
-        {
-            // Init
-            matStencil = MaterialManager::getSingleton().create(
-                "Ogre/StencilShadowVolumes",
-                ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
-            mShadowStencilPass = matStencil->getTechnique(0)->getPass(0);
-
-            if (ShadowVolumeExtrudeProgram::frgProgram->isSupported())
-            {
-
-                // Enable the finite point light extruder for now, just to get some params
-                mShadowStencilPass->setGpuProgram(
-                    GPT_VERTEX_PROGRAM, ShadowVolumeExtrudeProgram::get(Light::LT_POINT, true));
-                mShadowStencilPass->setGpuProgram(GPT_FRAGMENT_PROGRAM, ShadowVolumeExtrudeProgram::frgProgram);
-                msFiniteExtrusionParams =
-                    mShadowStencilPass->getVertexProgramParameters();
-                msFiniteExtrusionParams->setAutoConstant(0,
-                    GpuProgramParameters::ACT_WORLDVIEWPROJ_MATRIX);
-                msFiniteExtrusionParams->setAutoConstant(4,
-                    GpuProgramParameters::ACT_LIGHT_POSITION_OBJECT_SPACE);
-                // Note extra parameter
-                msFiniteExtrusionParams->setAutoConstant(5,
-                    GpuProgramParameters::ACT_SHADOW_EXTRUSION_DISTANCE);
-
-                try {
-                    msFiniteExtrusionParams->setNamedAutoConstant(
-                        "worldviewproj_matrix",
-                        GpuProgramParameters::ACT_WORLDVIEWPROJ_MATRIX);
-                    msFiniteExtrusionParams->setNamedAutoConstant(
-                        "light_position_object_space",
-                        GpuProgramParameters::ACT_LIGHT_POSITION_OBJECT_SPACE);
-                    msFiniteExtrusionParams->setNamedAutoConstant(
-                        "shadow_extrusion_distance",
-                        GpuProgramParameters::ACT_SHADOW_EXTRUSION_DISTANCE);
-                } catch(InvalidParametersException&) {} // ignore
-            }
-            matStencil->compile();
-            // Nothing else, we don't use this like a 'real' pass anyway,
-            // it's more of a placeholder
-        }
-        else
-        {
-            mShadowStencilPass = matStencil->getTechnique(0)->getPass(0);
-
-            if (!msFiniteExtrusionParams && mShadowStencilPass->isProgrammable())
-            {
-                msFiniteExtrusionParams = mShadowStencilPass->getVertexProgramParameters();
-            }
-        }
+        MaterialPtr matStencil = MaterialManager::getSingleton().getByName("Ogre/StencilShadowVolumes");
+        mShadowStencilPass = matStencil->getTechnique(0)->getPass(0);
+        msFiniteExtrusionParams = mShadowStencilPass->getVertexProgramParameters();
     }
 
-
-
-
     if (!mShadowModulativePass)
     {
-    MaterialPtr matModStencil = MaterialManager::getSingleton().getByName(
-        "Ogre/StencilShadowModulationPass");
-    if (!matModStencil)
-    {
-        // Init
-        matModStencil = MaterialManager::getSingleton().create(
-            "Ogre/StencilShadowModulationPass",
-            ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
-        mShadowModulativePass = matModStencil->getTechnique(0)->getPass(0);
-        mShadowModulativePass->setSceneBlending(SBF_DEST_COLOUR, SBF_ZERO);
-        mShadowModulativePass->setLightingEnabled(false);
-        mShadowModulativePass->setDepthWriteEnabled(false);
-        mShadowModulativePass->setDepthCheckEnabled(false);
-        mShadowModulativePass->createTextureUnitState();
-
-
-        mShadowModulativePass->setCullingMode(CULL_NONE);
-
-        mShadowModulativePass->setVertexProgram("Ogre/ShadowBlendVP");
-        mShadowModulativePass->setFragmentProgram("Ogre/ShadowBlendFP");
-
-        mShadowModulativePass->getFragmentProgramParameters()->setNamedConstant("shadowColor", mShadowColour);
+        MaterialPtr matModStencil = MaterialManager::getSingleton().getByName("Ogre/StencilShadowModulationPass");
         matModStencil->load();
-    }
-    else
-    {
         mShadowModulativePass = matModStencil->getTechnique(0)->getPass(0);
     }
 
-    }
-
     // Also init full screen quad while we're at it
     if (!mFullScreenQuad)
     {
         mFullScreenQuad = mSceneManager->createScreenSpaceRect();
-        mFullScreenQuad->setCorners(-1,1,1,-1);
     }
 
     // Also init shadow caster material for texture shadows
     if (!mShadowCasterPlainBlackPass)
     {
-        MaterialPtr matPlainBlack = MaterialManager::getSingleton().getByName(
-            "Ogre/TextureShadowCaster");
-        if (!matPlainBlack)
-        {
-            matPlainBlack = MaterialManager::getSingleton().create(
-                "Ogre/TextureShadowCaster",
-                ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
-            matPlainBlack->setReceiveShadows(false);
-            mShadowCasterPlainBlackPass = matPlainBlack->getTechnique(0)->getPass(0);
-            // Lighting has to be on, because we need shadow coloured objects
-            // Note that because we can't predict vertex programs, we'll have to
-            // bind light values to those, and so we bind White to ambient
-            // reflectance, and we'll set the ambient colour to the shadow colour
-            mShadowCasterPlainBlackPass->setAmbient(ColourValue::White);
-            mShadowCasterPlainBlackPass->setDiffuse(ColourValue::Black);
-            mShadowCasterPlainBlackPass->setSelfIllumination(ColourValue::Black);
-            mShadowCasterPlainBlackPass->setSpecular(ColourValue::Black);
-            // set depth bias in case this is used with PF_DEPTH
-            mShadowCasterPlainBlackPass->setDepthBias(-1, -1);
-            // Override fog
-            mShadowCasterPlainBlackPass->setFog(true, FOG_NONE);
-            // no textures or anything else, we will bind vertex programs
-            // every so often though
-        }
-        else
-        {
-            mShadowCasterPlainBlackPass = matPlainBlack->getTechnique(0)->getPass(0);
-        }
+        MaterialPtr matPlainBlack = MaterialManager::getSingleton().getByName("Ogre/TextureShadowCaster");
+        matPlainBlack->load();
+        mShadowCasterPlainBlackPass = matPlainBlack->getTechnique(0)->getPass(0);
     }
 
     if (!mShadowReceiverPass)
@@ -1649,13 +1478,12 @@ const Pass* SceneManager::ShadowRenderer::deriveShadowReceiverPass(const Pass* p
 
     if (mShadowTechnique & SHADOWDETAILTYPE_TEXTURE)
     {
-        Pass* retPass = NULL;
         if (pass->getParent()->getShadowReceiverMaterial())
         {
-            return retPass = pass->getParent()->getShadowReceiverMaterial()->getBestTechnique()->getPass(0);
+            return pass->getParent()->getShadowReceiverMaterial()->getBestTechnique()->getPass(0);
         }
 
-        retPass = mShadowTextureCustomReceiverPass ? mShadowTextureCustomReceiverPass : mShadowReceiverPass;
+        Pass* retPass = mShadowTextureCustomReceiverPass ? mShadowTextureCustomReceiverPass : mShadowReceiverPass;
 
         unsigned short keepTUCount;
         // If additive, need lighting parameters & standard programs
