@@ -45,7 +45,9 @@ namespace Ogre {
     {
         HighLevelGpuProgram::prepareImpl();
 
-        uint32 hash = _getHash();
+        const String& target = getTarget();
+        uint32 seed = FastHash(target.c_str(), target.length());
+        uint32 hash = _getHash(seed);
         if ( GpuProgramManager::getSingleton().isMicrocodeAvailableInCache(hash) )
         {
             getMicrocodeFromCache(hash);
@@ -99,6 +101,8 @@ namespace Ogre {
             cacheMicrocode->read( &def,  sizeof(GpuConstantDefinition));
 
             mParametersMap.emplace(paramName, def);
+
+            mLogicalToPhysical->bufferSize += def.elementSize * def.arraySize;
         }
     }
     //-----------------------------------------------------------------------
@@ -309,7 +313,6 @@ namespace Ogre {
             OGRE_LOCK_MUTEX(mLogicalToPhysical->mutex);
             mLogicalToPhysical->map.emplace(def.logicalIndex,
                     GpuLogicalIndexUse(def.physicalIndex, def.arraySize * def.elementSize, GPV_GLOBAL, def.isFloat() ? BCT_FLOAT : BCT_INT));
-            mLogicalToPhysical->bufferSize += def.arraySize * def.elementSize;
         }
         
     }
