@@ -29,7 +29,6 @@ THE SOFTWARE.
 #include "OgreGLSLExtSupport.h"
 #include "OgreGLSLLinkProgram.h"
 #include "OgreStringConverter.h"
-#include "OgreGLSLGpuProgram.h"
 #include "OgreGLSLProgram.h"
 #include "OgreGLSLLinkProgramManager.h"
 #include "OgreException.h"
@@ -77,10 +76,7 @@ namespace Ogre {
     }
 
     //-----------------------------------------------------------------------
-    GLSLLinkProgram::GLSLLinkProgram(GLSLProgram* vertexProgram, GLSLProgram* geometryProgram, GLSLProgram* fragmentProgram)
-        : GLSLProgramCommon(vertexProgram)
-        , mGeometryProgram(geometryProgram)
-        , mFragmentProgram(fragmentProgram)
+    GLSLLinkProgram::GLSLLinkProgram(const GLShaderList& shaders) : GLSLProgramCommon(shaders)
     {
         // Initialise uniform cache
         mUniformCache = new GLUniformCache();
@@ -89,7 +85,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     GLSLLinkProgram::~GLSLLinkProgram(void)
     {
-        glDeleteObjectARB(mGLProgramHandle);
+        glDeleteObjectARB((GLhandleARB)mGLProgramHandle);
 
         delete mUniformCache;
         mUniformCache = 0;
@@ -102,7 +98,7 @@ namespace Ogre {
         {           
             glGetError(); //Clean up the error. Otherwise will flood log.
 
-            mGLProgramHandle = glCreateProgramObjectARB();
+            mGLProgramHandle = (size_t)glCreateProgramObjectARB();
 
             GLenum glErr = glGetError();
             if(glErr != GL_NO_ERROR)
@@ -110,17 +106,11 @@ namespace Ogre {
                 reportGLSLError( glErr, "GLSLLinkProgram::activate", "Error Creating GLSL Program Object", 0 );
             }
 
-            uint32 hash = 0;
-            GpuProgram* progs[] = {mVertexShader, mGeometryProgram, mFragmentProgram};
-            for(auto p : progs)
-            {
-                if(!p) continue;
-                hash = p->_getHash(hash);
-            }
+            uint32 hash = getCombinedHash();
 
             if ( GpuProgramManager::getSingleton().canGetCompiledShaderBuffer() &&
                  GpuProgramManager::getSingleton().isMicrocodeAvailableInCache(hash) &&
-                 !mGeometryProgram)
+                 !mShaders[GPT_GEOMETRY_PROGRAM])
             {
                 getMicrocodeFromCache(hash);
             }
@@ -134,17 +124,10 @@ namespace Ogre {
         }
         if (mLinked)
         {
+            glUseProgramObjectARB( (GLhandleARB)mGLProgramHandle );
+
             GLenum glErr = glGetError();
             if(glErr != GL_NO_ERROR)
-            {
-                reportGLSLError( glErr, "GLSLLinkProgram::Activate",
-                    "Error prior to using GLSL Program Object : ", mGLProgramHandle, false, false);
-            }
-
-            glUseProgramObjectARB( mGLProgramHandle );
-
-            glErr = glGetError();
-            if(glErr != GL_NO_ERROR)
             {
                 reportGLSLError( glErr, "GLSLLinkProgram::Activate",
                     "Error using GLSL Program Object : ", mGLProgramHandle, false, false);
@@ -185,11 +168,19 @@ namespace Ogre {
         for (size_t i = 0; i < numAttribs; ++i)
         {
             const CustomAttribute& a = msCustomAttributes[i];
-            GLint attrib = glGetAttribLocationARB(mGLProgramHandle, a.name);
+            GLint attrib = glGetAttribLocationARB((GLhandleARB)mGLProgramHandle, a.name);
 
             if (attrib != -1)
             {
                 mValidAttributes.insert(a.attrib);
+
+                if(a.semantic != VES_TEXTURE_COORDINATES) continue;
+
+                // also enable next 4 attributes to allow matrix types in texcoord semantic
+                // might cause problems with mixing builtin and custom names,
+                // but then again you should not
+                for(int j = 0; j < 4; j++)
+                    mValidAttributes.insert(msCustomAttributes[i + j].attrib);
             }
         }
     }
@@ -206,17 +197,17 @@ namespace Ogre {
             const GpuConstantDefinitionMap* vertParams = 0;
             const GpuConstantDefinitionMap* fragParams = 0;
             const GpuConstantDefinitionMap* geomParams = 0;
-            if (mVertexShader)
+            if (mShaders[GPT_VERTEX_PROGRAM])
             {
-                vertParams = &(mVertexShader->getConstantDefinitions().map);
+                vertParams = &(mShaders[GPT_VERTEX_PROGRAM]->getConstantDefinitions().map);
             }
-            if (mGeometryProgram)
+            if (mShaders[GPT_GEOMETRY_PROGRAM])
             {
-                geomParams = &(mGeometryProgram->getConstantDefinitions().map);
+                geomParams = &(mShaders[GPT_GEOMETRY_PROGRAM]->getConstantDefinitions().map);
             }
-            if (mFragmentProgram)
+            if (mShaders[GPT_FRAGMENT_PROGRAM])
             {
-                fragParams = &(mFragmentProgram->getConstantDefinitions().map);
+                fragParams = &(mShaders[GPT_FRAGMENT_PROGRAM]->getConstantDefinitions().map);
             }
 
             GLSLLinkProgramManager::extractUniforms(
@@ -235,13 +226,7 @@ namespace Ogre {
         GLUniformReferenceIterator endUniform = mGLUniformReferences.end();
 
         // determine if we need to transpose matrices when binding
-        bool transpose = GL_TRUE;
-        if ((fromProgType == GPT_FRAGMENT_PROGRAM && mVertexShader && (!mVertexShader->getColumnMajorMatrices())) ||
-            (fromProgType == GPT_VERTEX_PROGRAM && mFragmentProgram && (!mFragmentProgram->getColumnMajorMatrices())) ||
-            (fromProgType == GPT_GEOMETRY_PROGRAM && mGeometryProgram && (!mGeometryProgram->getColumnMajorMatrices())))
-        {
-            transpose = GL_FALSE;
-        }
+        bool transpose = !mShaders[fromProgType] || mShaders[fromProgType]->getColumnMajorMatrices();
 
         for (;currentUniform != endUniform; ++currentUniform)
         {
@@ -256,33 +241,11 @@ namespace Ogre {
 
                     GLsizei glArraySize = (GLsizei)def->arraySize;
 
-                    bool shouldUpdate = true;
-
-                    switch (def->constType)
-                    {
-                        case GCT_INT1:
-                        case GCT_INT2:
-                        case GCT_INT3:
-                        case GCT_INT4:
-                        case GCT_SAMPLER1D:
-                        case GCT_SAMPLER1DSHADOW:
-                        case GCT_SAMPLER2D:
-                        case GCT_SAMPLER2DSHADOW:
-                        case GCT_SAMPLER2DARRAY:
-                        case GCT_SAMPLER3D:
-                        case GCT_SAMPLERCUBE:
-                            shouldUpdate = mUniformCache->updateUniform(currentUniform->mLocation,
-                                                                        params->getIntPointer(def->physicalIndex),
-                                                                        static_cast<GLsizei>(def->elementSize * def->arraySize * sizeof(int)));
-                            break;
-                        default:
-                            shouldUpdate = mUniformCache->updateUniform(currentUniform->mLocation,
-                                                                        params->getFloatPointer(def->physicalIndex),
-                                                                        static_cast<GLsizei>(def->elementSize * def->arraySize * sizeof(float)));
-                            break;
-
-                    }
+                    void* val = def->isSampler() ? (void*)params->getRegPointer(def->physicalIndex)
+                                                 : (void*)params->getFloatPointer(def->physicalIndex);
 
+                    bool shouldUpdate = mUniformCache->updateUniform(currentUniform->mLocation, val,
+                                                                     def->elementSize * def->arraySize * 4);
                     if(!shouldUpdate)
                         continue;
 
@@ -310,24 +273,24 @@ namespace Ogre {
                             transpose, params->getFloatPointer(def->physicalIndex));
                         break;
                     case GCT_MATRIX_2X3:
-                        if (GLEW_VERSION_2_1)
+                        if (GLAD_GL_VERSION_2_1)
                         {
                             glUniformMatrix2x3fv(currentUniform->mLocation, glArraySize, 
-                                transpose, params->getFloatPointer(def->physicalIndex));
+                                GL_FALSE, params->getFloatPointer(def->physicalIndex));
                         }
                         break;
                     case GCT_MATRIX_2X4:
-                        if (GLEW_VERSION_2_1)
+                        if (GLAD_GL_VERSION_2_1)
                         {
                             glUniformMatrix2x4fv(currentUniform->mLocation, glArraySize, 
-                                transpose, params->getFloatPointer(def->physicalIndex));
+                                GL_FALSE, params->getFloatPointer(def->physicalIndex));
                         }
                         break;
                     case GCT_MATRIX_3X2:
-                        if (GLEW_VERSION_2_1)
+                        if (GLAD_GL_VERSION_2_1)
                         {
                             glUniformMatrix3x2fv(currentUniform->mLocation, glArraySize, 
-                                transpose, params->getFloatPointer(def->physicalIndex));
+                                GL_FALSE, params->getFloatPointer(def->physicalIndex));
                         }
                         break;
                     case GCT_MATRIX_3X3:
@@ -335,33 +298,40 @@ namespace Ogre {
                             transpose, params->getFloatPointer(def->physicalIndex));
                         break;
                     case GCT_MATRIX_3X4:
-                        if (GLEW_VERSION_2_1)
+                        if (GLAD_GL_VERSION_2_1)
                         {
                             glUniformMatrix3x4fv(currentUniform->mLocation, glArraySize, 
-                                transpose, params->getFloatPointer(def->physicalIndex));
+                                GL_FALSE, params->getFloatPointer(def->physicalIndex));
                         }
                         break;
                     case GCT_MATRIX_4X2:
-                        if (GLEW_VERSION_2_1)
+                        if (GLAD_GL_VERSION_2_1)
                         {
                             glUniformMatrix4x2fv(currentUniform->mLocation, glArraySize, 
-                                transpose, params->getFloatPointer(def->physicalIndex));
+                                GL_FALSE, params->getFloatPointer(def->physicalIndex));
                         }
                         break;
                     case GCT_MATRIX_4X3:
-                        if (GLEW_VERSION_2_1)
+                        if (GLAD_GL_VERSION_2_1)
                         {
                             glUniformMatrix4x3fv(currentUniform->mLocation, glArraySize, 
-                                transpose, params->getFloatPointer(def->physicalIndex));
+                                GL_FALSE, params->getFloatPointer(def->physicalIndex));
                         }
                         break;
                     case GCT_MATRIX_4X4:
                         glUniformMatrix4fvARB(currentUniform->mLocation, glArraySize, 
                             transpose, params->getFloatPointer(def->physicalIndex));
                         break;
+                    case GCT_SAMPLER1D:
+                    case GCT_SAMPLER1DSHADOW:
+                    case GCT_SAMPLER2D:
+                    case GCT_SAMPLER2DSHADOW:
+                    case GCT_SAMPLER2DARRAY:
+                    case GCT_SAMPLER3D:
+                    case GCT_SAMPLERCUBE:
+                        // samplers handled like 1-element ints
                     case GCT_INT1:
-                        glUniform1ivARB(currentUniform->mLocation, glArraySize, 
-                            (GLint*)params->getIntPointer(def->physicalIndex));
+                        glUniform1ivARB(currentUniform->mLocation, glArraySize, (GLint*)val);
                         break;
                     case GCT_INT2:
                         glUniform2ivARB(currentUniform->mLocation, glArraySize, 
@@ -375,17 +345,6 @@ namespace Ogre {
                         glUniform4ivARB(currentUniform->mLocation, glArraySize, 
                             (GLint*)params->getIntPointer(def->physicalIndex));
                         break;
-                    case GCT_SAMPLER1D:
-                    case GCT_SAMPLER1DSHADOW:
-                    case GCT_SAMPLER2D:
-                    case GCT_SAMPLER2DSHADOW:
-                                        case GCT_SAMPLER2DARRAY:
-                    case GCT_SAMPLER3D:
-                    case GCT_SAMPLERCUBE:
-                        // samplers handled like 1-element ints
-                        glUniform1ivARB(currentUniform->mLocation, 1, 
-                            (GLint*)params->getIntPointer(def->physicalIndex));
-                        break;
                     case GCT_UNKNOWN:
                     default:
                         break;
@@ -404,35 +363,13 @@ namespace Ogre {
         } // end for
     }
     //-----------------------------------------------------------------------
-    Ogre::String GLSLLinkProgram::getCombinedName()
-    {
-        String name;
-        if (mVertexShader)
-        {
-            name += "Vertex Program:" ;
-            name += mVertexShader->getName();
-        }
-        if (mFragmentProgram)
-        {
-            name += " Fragment Program:" ;
-            name += mFragmentProgram->getName();
-        }
-        if (mGeometryProgram)
-        {
-            name += " Geometry Program:" ;
-            name += mGeometryProgram->getName();
-        }
-        return name;
-    }
-    //-----------------------------------------------------------------------
     void GLSLLinkProgram::compileAndLink()
     {
         uint32 hash = 0;
-        if (mVertexShader)
+        if (mShaders[GPT_VERTEX_PROGRAM])
         {
             // attach Vertex Program
-            mVertexShader->attachToProgramObject(mGLProgramHandle);
-            setSkeletalAnimationIncluded(mVertexShader->isSkeletalAnimationIncluded());
+            mShaders[GPT_VERTEX_PROGRAM]->attachToProgramObject(mGLProgramHandle);
 
             // Some drivers (e.g. OS X on nvidia) incorrectly determine the attribute binding automatically
 
@@ -444,9 +381,9 @@ namespace Ogre {
             // until it is linked (chicken and egg!) we have to parse the source
 
             size_t numAttribs = sizeof(msCustomAttributes)/sizeof(CustomAttribute);
-            const String& vpSource = mVertexShader->getSource();
+            const String& vpSource = mShaders[GPT_VERTEX_PROGRAM]->getSource();
             
-            hash = mVertexShader->_getHash(hash);
+            hash = mShaders[GPT_VERTEX_PROGRAM]->_getHash(hash);
             for (size_t i = 0; i < numAttribs; ++i)
             {
                 const CustomAttribute& a = msCustomAttributes[i];
@@ -470,7 +407,7 @@ namespace Ogre {
                         StringVector vec = StringUtil::split(expr);
                         if ((vec[0] == "in" || vec[0] == "attribute") && vec[2] == a.name)
                         {
-                            glBindAttribLocationARB(mGLProgramHandle, a.attrib, a.name);
+                            glBindAttribLocationARB((GLhandleARB)mGLProgramHandle, a.attrib, a.name);
                             foundAttr = true;
                         }
                     }
@@ -480,39 +417,39 @@ namespace Ogre {
             }
         }
 
-        if (mGeometryProgram)
+        if (auto gshader = static_cast<GLSLProgram*>(mShaders[GPT_GEOMETRY_PROGRAM]))
         {
-            hash = mGeometryProgram->_getHash(hash);
+            hash = mShaders[GPT_GEOMETRY_PROGRAM]->_getHash(hash);
             // attach Geometry Program
-            mGeometryProgram->attachToProgramObject(mGLProgramHandle);
+            mShaders[GPT_GEOMETRY_PROGRAM]->attachToProgramObject(mGLProgramHandle);
 
             //Don't set adjacency flag. We handle it internally and expose "false"
 
-            RenderOperation::OperationType inputOperationType = mGeometryProgram->getInputOperationType();
+            RenderOperation::OperationType inputOperationType = gshader->getInputOperationType();
             glProgramParameteriEXT(mGLProgramHandle, GL_GEOMETRY_INPUT_TYPE_EXT,
                 getGLGeometryInputPrimitiveType(inputOperationType));
 
-            RenderOperation::OperationType outputOperationType = mGeometryProgram->getOutputOperationType();
+            RenderOperation::OperationType outputOperationType = gshader->getOutputOperationType();
 
             glProgramParameteriEXT(mGLProgramHandle, GL_GEOMETRY_OUTPUT_TYPE_EXT,
                 getGLGeometryOutputPrimitiveType(outputOperationType));
 
             glProgramParameteriEXT(mGLProgramHandle, GL_GEOMETRY_VERTICES_OUT_EXT,
-                mGeometryProgram->getMaxOutputVertices());
+                gshader->getMaxOutputVertices());
         }
 
-        if (mFragmentProgram)
+        if (mShaders[GPT_FRAGMENT_PROGRAM])
         {
-            hash = mFragmentProgram->_getHash(hash);
+            hash = mShaders[GPT_FRAGMENT_PROGRAM]->_getHash(hash);
             // attach Fragment Program
-            mFragmentProgram->attachToProgramObject(mGLProgramHandle);
+            mShaders[GPT_FRAGMENT_PROGRAM]->attachToProgramObject(mGLProgramHandle);
         }
 
         
         // now the link
 
-        glLinkProgramARB( mGLProgramHandle );
-        glGetObjectParameterivARB( mGLProgramHandle, GL_OBJECT_LINK_STATUS_ARB, &mLinked );
+        glLinkProgramARB( (GLhandleARB)mGLProgramHandle );
+        glGetObjectParameterivARB( (GLhandleARB)mGLProgramHandle, GL_OBJECT_LINK_STATUS_ARB, &mLinked );
 
         // force logging and raise exception if not linked
         GLenum glErr = glGetError();
